@@ -1,31 +1,20 @@
-"""
-Report
-======
-
-The main routine containing the `Report` class.
-
-"""
+"""The main module containing the `Report` class."""
 
 import importlib
-import multiprocessing
-from pathlib import Path
-import platform
 import sys
-import textwrap
 import time
 from types import ModuleType
 
 from .knowledge import (
-    FILESYSTEM_TYPE,
-    MKL_INFO,
-    TOTAL_RAM,
     VERSION_ATTRIBUTES,
     VERSION_METHODS,
+    get_filesystem_type,
     in_ipykernel,
     in_ipython,
 )
 
-MODULE_NOT_FOUND = 'Could not import'
+MODULE_NOT_FOUND = 'Module not found'
+MODULE_TROUBLE = 'Trouble importing'
 VERSION_NOT_FOUND = 'Version unknown'
 
 
@@ -35,32 +24,39 @@ class PlatformInfo:
 
     @property
     def system(self):
-        """Returns the system/OS name.
+        """Return the system/OS name.
+
         E.g. ``'Linux'``, ``'Windows'``, or ``'Java'``. An empty string is
         returned if the value cannot be determined.
         """
-        return platform.system()
+        return platform().system()
 
     @property
     def platform(self):
-        return platform.platform()
+        """Return the platform."""
+        return platform().platform()
 
     @property
     def machine(self):
-        """Returns the machine type, e.g. 'i386'
+        """Return the machine type, e.g. 'i386'.
+
         An empty string is returned if the value cannot be determined.
         """
-        return platform.machine()
+        return platform().machine()
 
     @property
     def architecture(self):
-        """bit architecture used for the executable"""
-        return platform.architecture()[0]
+        """Return the bit architecture used for the executable."""
+        return platform().architecture()[0]
 
     @property
     def cpu_count(self):
         """Return the number of CPUs in the system."""
-        return multiprocessing.cpu_count()
+        if not hasattr(self, '_cpu_count'):
+            import multiprocessing  # lazy-load see PR#85
+
+            self._cpu_count = multiprocessing.cpu_count()
+        return self._cpu_count
 
     @property
     def total_ram(self):
@@ -68,26 +64,66 @@ class PlatformInfo:
 
         If not available, returns 'unknown'.
         """
-        if TOTAL_RAM:
-            return TOTAL_RAM
-        return 'unknown'
+        if not hasattr(self, '_total_ram'):
+
+            try:
+                import psutil  # lazy-load see PR#85
+
+                tmem = psutil.virtual_memory().total
+                self._total_ram = '{:.1f} GiB'.format(tmem / (1024.0**3))
+            except ImportError:
+                self._total_ram = 'unknown'
+
+        return self._total_ram
+
+    @property
+    def mkl_info(self):
+        """Return MKL info.
+
+        If not available, returns 'unknown'.
+        """
+        if not hasattr(self, '_mkl_info'):
+            try:
+                import mkl  # lazy-load see PR#85
+
+                mkl.get_version_string()
+            except (ImportError, AttributeError):
+                mkl = False
+
+            try:
+                import numexpr  # lazy-load see PR#85
+
+            except ImportError:
+                numexpr = False
+
+            # Get mkl info from numexpr or mkl, if available
+            if mkl:
+                self._mkl_info = mkl.get_version_string()
+            elif numexpr:
+                self._mkl_info = numexpr.get_vml_version()
+            else:
+                self._mkl_info = None
+
+        return self._mkl_info
 
     @property
     def date(self):
+        """Return the date formatted as a string."""
         return time.strftime('%a %b %d %H:%M:%S %Y %Z')
 
     @property
     def filesystem(self):
-        """Get the type of the file system at the path of the scooby package"""
-        if FILESYSTEM_TYPE:
-            return FILESYSTEM_TYPE
-        return 'unknown'
+        """Get the type of the file system at the path of the scooby package."""
+        if not hasattr(self, '_filesystem'):
+            self._filesystem = get_filesystem_type()
+        return self._filesystem
 
 
 class PythonInfo:
     """Internal helper class to access Python info and package versions."""
 
     def __init__(self, additional, core, optional, sort):
+        """Initialize python info."""
         self._packages = {}  # Holds name of packages and their version
         self._sort = sort
 
@@ -98,7 +134,6 @@ class PythonInfo:
 
     def _add_packages(self, packages, optional=False):
         """Add all packages to list; optional ones only if available."""
-
         # Ensure arguments are a list
         if isinstance(packages, (str, ModuleType)):
             pckgs = [
@@ -117,10 +152,12 @@ class PythonInfo:
 
     @property
     def sys_version(self):
+        """Return the system version."""
         return sys.version
 
     @property
     def python_environment(self):
+        """Return the python environment."""
         if in_ipykernel():
             return 'Jupyter'
         elif in_ipython():
@@ -129,8 +166,10 @@ class PythonInfo:
 
     @property
     def packages(self):
-        """Return versions of all packages
-        (available and unavailable/unknown)
+        """Return versions of all packages.
+
+        Includes available and unavailable/unknown.
+
         """
         pckg_dict = dict(self._packages)
         if self._sort:
@@ -159,20 +198,23 @@ class Report(PlatformInfo, PythonInfo):
     optional : list(ModuleType), list(str)
         A list of packages to list if they are available. If not available,
         no warnings or error will be thrown.
-        Defaults to ['numpy', 'scipy', 'IPython', 'matplotlib', 'scooby']
+        Defaults to ``['numpy', 'scipy', 'IPython', 'matplotlib', 'scooby']``
 
     ncol : int, optional
         Number of package-columns in html table (no effect in text-version);
         Defaults to 3.
 
     text_width : int, optional
-        The text width for non-HTML display modes
+        The text width for non-HTML display modes.
 
     sort : bool, optional
-        Sort the packages when the report is shown
+        Sort the packages when the report is shown.
 
-    extra_meta : tuple(str, str)
-        Additional two component pairs of meta information to display
+    extra_meta : tuple(str, str), optional
+        Additional two component pairs of meta information to display.
+
+    max_width : int, optional
+        Max-width of html-table. By default None.
 
     """
 
@@ -185,8 +227,9 @@ class Report(PlatformInfo, PythonInfo):
         text_width=80,
         sort=False,
         extra_meta=None,
+        max_width=None,
     ):
-
+        """Initialize report."""
         # Set default optional packages to investigate
         if optional is None:
             optional = ['numpy', 'scipy', 'IPython', 'matplotlib', 'scooby']
@@ -194,6 +237,7 @@ class Report(PlatformInfo, PythonInfo):
         PythonInfo.__init__(self, additional=additional, core=core, optional=optional, sort=sort)
         self.ncol = int(ncol)
         self.text_width = int(text_width)
+        self.max_width = max_width
 
         if extra_meta is not None:
             if not isinstance(extra_meta, (list, tuple)):
@@ -208,7 +252,8 @@ class Report(PlatformInfo, PythonInfo):
         self._extra_meta = extra_meta
 
     def __repr__(self):
-        """Plain-text version information."""
+        """Return Plain-text version information."""
+        import textwrap  # lazy-load see PR#85
 
         # Width for text-version
         text = '\n' + self.text_width * '-' + '\n'
@@ -223,9 +268,12 @@ class Report(PlatformInfo, PythonInfo):
         text += date_text + '\n'
 
         # Get length of longest package: min of 18 and max of 40
-        row_width = min(40, max(18, len(max(self._packages.keys(), key=len))))
+        if self._packages:
+            row_width = min(40, max(18, len(max(self._packages.keys(), key=len))))
+        else:
+            row_width = 18
 
-        # ########## Platform/OS details ############
+        # Platform/OS details
         repr_dict = self.to_dict()
         for key in ['OS', 'CPU(s)', 'Machine', 'Architecture', 'RAM', 'Environment', 'File system']:
             if key in repr_dict:
@@ -233,55 +281,57 @@ class Report(PlatformInfo, PythonInfo):
         for key, value in self._extra_meta:
             text += f'{key:>{row_width}} : {value}\n'
 
-        # ########## Python details ############
+        # Python details
         text += '\n'
         for txt in textwrap.wrap('Python ' + self.sys_version, self.text_width - 4):
             text += '  ' + txt + '\n'
-        text += '\n'
+        if self._packages:
+            text += '\n'
 
         # Loop over packages
         for name, version in self._packages.items():
             text += f'{name:>{row_width}} : {version}\n'
 
-        # ########## MKL details ############
-        if MKL_INFO:
+        # MKL details
+        if self.mkl_info:
             text += '\n'
-            for txt in textwrap.wrap(MKL_INFO, self.text_width - 4):
+            for txt in textwrap.wrap(self.mkl_info, self.text_width - 4):
                 text += '  ' + txt + '\n'
 
-        # ########## Finish ############
+        # Finish
         text += self.text_width * '-'
 
         return text
 
     def _repr_html_(self):
-        """HTML-rendered version information."""
-
+        """Return HTML-rendered version information."""
         # Define html-styles
-        border = "border: 2px solid #fff;'"
+        border = "border: 1px solid;'"
 
         def colspan(html, txt, ncol, nrow):
             r"""Print txt in a row spanning whole table."""
             html += "  <tr>\n"
-            html += "     <td style='text-align: center; "
+            html += "     <td style='"
+            if ncol == 1:
+                html += "text-align: left; "
+            else:
+                html += "text-align: center; "
             if nrow == 0:
                 html += "font-weight: bold; font-size: 1.2em; "
-            elif nrow % 2 == 0:
-                html += "background-color: #ddd;"
             html += border + " colspan='"
-            html += str(2 * ncol) + "'>%s</td>\n" % txt
+            html += f"{2 * ncol}'>{txt}</td>\n"
             html += "  </tr>\n"
             return html
 
         def cols(html, version, name, ncol, i):
             r"""Print package information in two cells."""
-
             # Check if we have to start a new row
             if i > 0 and i % ncol == 0:
                 html += "  </tr>\n"
                 html += "  <tr>\n"
 
-            html += "    <td style='text-align: right; background-color: #ccc;"
+            align = "left" if ncol == 1 else "right"
+            html += f"    <td style='text-align: {align};"
             html += " " + border + ">%s</td>\n" % name
 
             html += "    <td style='text-align: left; "
@@ -290,12 +340,15 @@ class Report(PlatformInfo, PythonInfo):
             return html, i + 1
 
         # Start html-table
-        html = "<table style='border: 3px solid #ddd;'>\n"
+        html = "<table style='border: 1.5px solid;"
+        if self.max_width:
+            html += f" max-width: {self.max_width}px;"
+        html += "'>\n"
 
         # Date and time info as title
         html = colspan(html, self.date, self.ncol, 0)
 
-        # ########## Platform/OS details ############
+        # Platform/OS details
         html += "  <tr>\n"
         repr_dict = self.to_dict()
         i = 0
@@ -307,10 +360,10 @@ class Report(PlatformInfo, PythonInfo):
         # Finish row
         html += "  </tr>\n"
 
-        # ########## Python details ############
+        # Python details
         html = colspan(html, 'Python ' + self.sys_version, self.ncol, 1)
-
         html += "  <tr>\n"
+
         # Loop over packages
         i = 0  # Reset count for rows.
         for name, version in self.packages.items():
@@ -323,18 +376,17 @@ class Report(PlatformInfo, PythonInfo):
         # Finish row
         html += "  </tr>\n"
 
-        # ########## MKL details ############
-        if MKL_INFO:
-            html = colspan(html, MKL_INFO, self.ncol, 2)
+        # MKL details
+        if self.mkl_info:
+            html = colspan(html, self.mkl_info, self.ncol, 2)
 
-        # ########## Finish ############
+        # Finish
         html += "</table>"
 
         return html
 
     def to_dict(self):
         """Return report as dict for storage."""
-
         out = {}
 
         # Date and time info
@@ -345,9 +397,9 @@ class Report(PlatformInfo, PythonInfo):
         out['CPU(s)'] = str(self.cpu_count)
         out['Machine'] = self.machine
         out['Architecture'] = self.architecture
-        if FILESYSTEM_TYPE:
+        if self.filesystem:
             out['File system'] = self.filesystem
-        if TOTAL_RAM:
+        if self.total_ram != 'unknown':
             out['RAM'] = self.total_ram
         out['Environment'] = self.python_environment
         for meta in self._extra_meta:
@@ -361,16 +413,15 @@ class Report(PlatformInfo, PythonInfo):
             out[name] = version
 
         # MKL details
-        if MKL_INFO:
-            out['MKL'] = MKL_INFO
+        if self.mkl_info:
+            out['MKL'] = self.mkl_info
 
         return out
 
 
 # This functionaliy might also be of interest on its own.
 def get_version(module):
-    """Get the version of `module` by passing the package or it's name.
-
+    """Get the version of ``module`` by passing the package or it's name.
 
     Parameters
     ----------
@@ -386,7 +437,6 @@ def get_version(module):
     version : str or None
         Version of module.
     """
-
     # module is (1) a string or (2) a module.
     # If (1), we have to load it, if (2), we have to get its name.
     if isinstance(module, str):  # Case 1: module is a string; import
@@ -397,6 +447,8 @@ def get_version(module):
             module = importlib.import_module(name)
         except ImportError:
             module = None
+        except:  # noqa
+            return name, MODULE_TROUBLE
 
     elif isinstance(module, ModuleType):  # Case 2: module is module; get name
         name = module.__name__
@@ -432,3 +484,10 @@ def get_version(module):
 
         # If not found, return VERSION_NOT_FOUND
         return name, VERSION_NOT_FOUND
+
+
+def platform():
+    """Return platform as lazy load; see PR#85."""
+    import platform
+
+    return platform
